#!perl -w

use strict;
our $VERSION = '0.01'; # M::B::ModuleInfo should see this version in 'main'
                       # when it's not the first non-empty line in the
                       # assumed package main.

print "Hello, world";

__END__

=head1 NAME

sample.pl - sample script to test manifypods action

=cut
